@@ -1,9 +1,9 @@
 //
-//  file:  repattrn.cpp    
+//  file:  repattrn.cpp
 //
 /*
 ***************************************************************************
-*   Copyright (C) 2002-2006 International Business Machines Corporation   *
+*   Copyright (C) 2002-2008 International Business Machines Corporation   *
 *   and others. All rights reserved.                                      *
 ***************************************************************************
 */
@@ -46,7 +46,7 @@ RegexPattern::RegexPattern() {
 //
 //--------------------------------------------------------------------------
 RegexPattern::RegexPattern(const RegexPattern &other) :  UObject(other) {
-    init(); 
+    init();
     *this = other;
 }
 
@@ -78,9 +78,9 @@ RegexPattern &RegexPattern::operator = (const RegexPattern &other) {
     fFrameSize        = other.fFrameSize;
     fDataSize         = other.fDataSize;
     fMaxCaptureDigits = other.fMaxCaptureDigits;
-    fStaticSets       = other.fStaticSets; 
+    fStaticSets       = other.fStaticSets;
     fStaticSets8      = other.fStaticSets8;
-    
+
     fStartType        = other.fStartType;
     fInitialStringIdx = other.fInitialStringIdx;
     fInitialStringLen = other.fInitialStringLen;
@@ -92,13 +92,17 @@ RegexPattern &RegexPattern::operator = (const RegexPattern &other) {
     fCompiledPat->assign(*other.fCompiledPat, fDeferredStatus);
     fGroupMap->assign(*other.fGroupMap, fDeferredStatus);
 
-    //  Copy the Unicode Sets.  
+    //  Copy the Unicode Sets.
     //    Could be made more efficient if the sets were reference counted and shared,
-    //    but I doubt that pattern copying will be particularly common. 
+    //    but I doubt that pattern copying will be particularly common.
     //    Note:  init() already added an empty element zero to fSets
     int32_t i;
     int32_t  numSets = other.fSets->size();
     fSets8 = new Regex8BitSet[numSets];
+    if (fSets8 == NULL) {
+    	fDeferredStatus = U_MEMORY_ALLOCATION_ERROR;
+    	return *this;
+    }
     for (i=1; i<numSets; i++) {
         if (U_FAILURE(fDeferredStatus)) {
             return *this;
@@ -135,7 +139,7 @@ void RegexPattern::init() {
     fFrameSize        = 0;
     fDataSize         = 0;
     fGroupMap         = NULL;
-    fMaxCaptureDigits = 1;  
+    fMaxCaptureDigits = 1;
     fStaticSets       = NULL;
     fStaticSets8      = NULL;
     fStartType        = START_NO_INFO;
@@ -144,7 +148,7 @@ void RegexPattern::init() {
     fInitialChars     = NULL;
     fInitialChar      = 0;
     fInitialChars8    = NULL;
-    
+
     fCompiledPat      = new UVector32(fDeferredStatus);
     fGroupMap         = new UVector32(fDeferredStatus);
     fSets             = new UVector(fDeferredStatus);
@@ -166,7 +170,7 @@ void RegexPattern::init() {
 
 //--------------------------------------------------------------------------
 //
-//   zap            Delete everything owned by this RegexPattern. 
+//   zap            Delete everything owned by this RegexPattern.
 //
 //--------------------------------------------------------------------------
 void RegexPattern::zap() {
@@ -208,7 +212,7 @@ RegexPattern::~RegexPattern() {
 //   Clone
 //
 //--------------------------------------------------------------------------
-RegexPattern  *RegexPattern::clone() const { 
+RegexPattern  *RegexPattern::clone() const {
     RegexPattern  *copy = new RegexPattern(*this);
     return copy;
 }
@@ -229,7 +233,7 @@ UBool   RegexPattern::operator ==(const RegexPattern &other) const {
 
 //---------------------------------------------------------------------
 //
-//   compile        
+//   compile
 //
 //---------------------------------------------------------------------
 RegexPattern * U_EXPORT2
@@ -244,7 +248,8 @@ RegexPattern::compile(const UnicodeString &regex,
     }
 
     const uint32_t allFlags = UREGEX_CANON_EQ | UREGEX_CASE_INSENSITIVE | UREGEX_COMMENTS |
-                              UREGEX_DOTALL   | UREGEX_MULTILINE | UREGEX_UWORD;
+                              UREGEX_DOTALL   | UREGEX_MULTILINE        | UREGEX_UWORD |
+                              UREGEX_ERROR_ON_UNKNOWN_ESCAPES           | UREGEX_UNIX_LINES;
 
     if ((flags & ~allFlags) != 0) {
         status = U_REGEX_INVALID_FLAG;
@@ -263,25 +268,31 @@ RegexPattern::compile(const UnicodeString &regex,
     }
     if (U_FAILURE(This->fDeferredStatus)) {
         status = This->fDeferredStatus;
+        delete This;
         return NULL;
     }
     This->fFlags = flags;
 
     RegexCompile     compiler(This, status);
     compiler.compile(regex, pe, status);
+    
+    if (U_FAILURE(status)) {
+        delete This;
+        This = NULL;
+    }
 
     return This;
 }
-    
+
 //
 //   compile with default flags.
 //
 RegexPattern * U_EXPORT2
 RegexPattern::compile(const UnicodeString &regex,
                       UParseError         &pe,
-                      UErrorCode          &err) 
+                      UErrorCode          &err)
 {
-    return compile(regex, 0, pe, err); 
+    return compile(regex, 0, pe, err);
 }
 
 
@@ -292,10 +303,10 @@ RegexPattern::compile(const UnicodeString &regex,
 RegexPattern * U_EXPORT2
 RegexPattern::compile( const UnicodeString &regex,
         uint32_t             flags,
-        UErrorCode           &err) 
+        UErrorCode           &err)
 {
     UParseError pe;
-    return compile(regex, flags, pe, err); 
+    return compile(regex, flags, pe, err);
 }
 
 
@@ -318,6 +329,7 @@ uint32_t RegexPattern::flags() const {
 RegexMatcher *RegexPattern::matcher(const UnicodeString &input,
                                     UErrorCode          &status)  const {
     RegexMatcher    *retMatcher = matcher(status);
+    retMatcher->fDeferredStatus = status;
     if (retMatcher != NULL) {
         retMatcher->reset(input);
     }
@@ -326,7 +338,7 @@ RegexMatcher *RegexPattern::matcher(const UnicodeString &input,
 
 #if 0
 RegexMatcher *RegexPattern::matcher(const UChar * /*input*/,
-                                    UErrorCode          &status)  const 
+                                    UErrorCode          &status)  const
 {
     /* This should never get called. The API with UnicodeString should be called instead. */
     if (U_SUCCESS(status)) {
@@ -352,7 +364,7 @@ RegexMatcher *RegexPattern::matcher(UErrorCode &status)  const {
         return NULL;
     }
 
-    retMatcher = new RegexMatcher(this); 
+    retMatcher = new RegexMatcher(this);
     if (retMatcher == NULL) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return NULL;
@@ -418,7 +430,11 @@ int32_t  RegexPattern::split(const UnicodeString &input,
     };
 
     RegexMatcher  m(this);
-    int32_t r = m.split(input, dest, destCapacity, status);
+    int32_t r = 0;
+    // Check m's status to make sure all is ok.
+    if (U_SUCCESS(m.fDeferredStatus)) {
+    	r = m.split(input, dest, destCapacity, status);
+    }
     return r;
 }
 
@@ -437,17 +453,15 @@ void   RegexPattern::dumpOp(int32_t index) const {
     int32_t val         = URX_VAL(op);
     int32_t type        = URX_TYPE(op);
     int32_t pinnedType  = type;
-    if (pinnedType >= sizeof(opNames)/sizeof(char *)) {
+    if ((uint32_t)pinnedType >= sizeof(opNames)/sizeof(char *)) {
         pinnedType = 0;
     }
-    
+
     REGEX_DUMP_DEBUG_PRINTF(("%4d   %08x    %-15s  ", index, op, opNames[pinnedType]));
     switch (type) {
     case URX_NOP:
     case URX_DOTANY:
     case URX_DOTANY_ALL:
-    case URX_DOTANY_PL:
-    case URX_DOTANY_ALL_PL:
     case URX_FAIL:
     case URX_CARET:
     case URX_DOLLAR:
@@ -458,7 +472,7 @@ void   RegexPattern::dumpOp(int32_t index) const {
     case URX_CARET_M:
         // Types with no operand field of interest.
         break;
-        
+
     case URX_RESERVED_OP:
     case URX_START_CAPTURE:
     case URX_END_CAPTURE:
@@ -494,12 +508,12 @@ void   RegexPattern::dumpOp(int32_t index) const {
         // types with an integer operand field.
         REGEX_DUMP_DEBUG_PRINTF(("%d", val));
         break;
-        
+
     case URX_ONECHAR:
     case URX_ONECHAR_I:
         REGEX_DUMP_DEBUG_PRINTF(("%c", val<256?val:'?'));
         break;
-        
+
     case URX_STRING:
     case URX_STRING_I:
         {
@@ -543,7 +557,7 @@ void   RegexPattern::dumpOp(int32_t index) const {
         }
         break;
 
-        
+
     default:
         REGEX_DUMP_DEBUG_PRINTF(("??????"));
         break;
@@ -554,7 +568,7 @@ void   RegexPattern::dumpOp(int32_t index) const {
 
 
 #if defined(REGEX_DEBUG)
-U_CAPI void  U_EXPORT2 
+U_CAPI void  U_EXPORT2
 RegexPatternDump(const RegexPattern *This) {
     int      index;
     int      i;
@@ -565,7 +579,7 @@ RegexPatternDump(const RegexPattern *This) {
     }
     REGEX_DUMP_DEBUG_PRINTF(("\n"));
     REGEX_DUMP_DEBUG_PRINTF(("   Min Match Length:  %d\n", This->fMinMatchLen));
-    REGEX_DUMP_DEBUG_PRINTF(("   Match Start Type:  %s\n", START_OF_MATCH_STR(This->fStartType)));   
+    REGEX_DUMP_DEBUG_PRINTF(("   Match Start Type:  %s\n", START_OF_MATCH_STR(This->fStartType)));
     if (This->fStartType == START_STRING) {
         REGEX_DUMP_DEBUG_PRINTF(("    Initial match sting: \""));
         for (i=This->fInitialStringIdx; i<This->fInitialStringIdx+This->fInitialStringLen; i++) {
@@ -580,7 +594,7 @@ RegexPatternDump(const RegexPattern *This) {
         REGEX_DUMP_DEBUG_PRINTF(("     Match First Chars : "));
         for (i=0; i<numSetChars; i++) {
             UChar32 c = This->fInitialChars->charAt(i);
-            if (0x20<c && c <0x7e) { 
+            if (0x20<c && c <0x7e) {
                 REGEX_DUMP_DEBUG_PRINTF(("%c ", c));
             } else {
                 REGEX_DUMP_DEBUG_PRINTF(("%#x ", c));
@@ -606,7 +620,7 @@ RegexPatternDump(const RegexPattern *This) {
         This->dumpOp(index);
     }
     REGEX_DUMP_DEBUG_PRINTF(("\n\n"));
-};
+}
 #endif
 
 
